'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var reduxSaga = require('redux-saga');
var reduxSaga_effects = require('redux-saga/effects');
var ramda = require('ramda');

/**
 * OTP
 *
 * Docs {@link http://erlang.org/doc/design_principles/des_princ.html Here}.
 * @namespace OTP
 *
 */

/* eslint-disable no-continue */

var ONE_FOR_ONE = 'ONE_FOR_ONE';
var ONE_FOR_MANY = 'ONE_FOR_MANY';
var PERMANENT = 'PERMANENT';
var TEMPORARY = 'TEMPORARY';

var debug = require('debug');

/**
 * Supervisor OTP behavior
 *
 * Docs {@link http://erlang.org/documentation/doc-4.9.1/doc/design_principles/sup_princ.html Here}.
 * @memberof OTP
 *
 */
var supervisor = {
  debug: debug('otp/supervisor'),

  start_link: regeneratorRuntime.mark(function start_link(task, id) {
    return regeneratorRuntime.wrap(function start_link$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            supervisor.debug('start_link', { task: task, id: id });
            _context.next = 3;
            return reduxSaga_effects.call(task);

          case 3:
            return _context.abrupt('return', id);

          case 4:
          case 'end':
            return _context.stop();
        }
      }
    }, start_link, this);
  }),
  init: regeneratorRuntime.mark(function init(_ref) {
    var _ref$supFlags = _ref.supFlags,
        supFlags = _ref$supFlags === undefined ? {} : _ref$supFlags,
        _ref$childSpecs = _ref.childSpecs,
        childSpecs = _ref$childSpecs === undefined ? [] : _ref$childSpecs;

    var _supFlags$strategy, strategy, _supFlags$maxR, maxR, _supFlags$maxT, maxT, children, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, _child, id, start, _child$restart, restart, task, tasks, waitedLongEnough, terminatedId, _iteratorNormalCompletion2, _didIteratorError2, _iteratorError2, _iterator2, _step2, child;

    return regeneratorRuntime.wrap(function init$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _supFlags$strategy = supFlags.strategy, strategy = _supFlags$strategy === undefined ? ONE_FOR_ONE : _supFlags$strategy, _supFlags$maxR = supFlags.maxR, maxR = _supFlags$maxR === undefined ? 1 : _supFlags$maxR, _supFlags$maxT = supFlags.maxT, maxT = _supFlags$maxT === undefined ? 1000 : _supFlags$maxT;


            supervisor.debug('init', { supFlags: supFlags, childSpecs: childSpecs });
            children = {};
            _iteratorNormalCompletion = true;
            _didIteratorError = false;
            _iteratorError = undefined;
            _context2.prev = 6;
            _iterator = childSpecs[Symbol.iterator]();

          case 8:
            if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
              _context2.next = 19;
              break;
            }

            _child = _step.value;
            id = _child.id, start = _child.start, _child$restart = _child.restart, restart = _child$restart === undefined ? PERMANENT : _child$restart;

            // fork worker

            _context2.next = 13;
            return reduxSaga_effects.fork(start);

          case 13:
            task = _context2.sent;

            supervisor.debug('init: forked', id);

            children[id] = { id: id, task: task, start: start, restart: restart, terminated: false };

          case 16:
            _iteratorNormalCompletion = true;
            _context2.next = 8;
            break;

          case 19:
            _context2.next = 25;
            break;

          case 21:
            _context2.prev = 21;
            _context2.t0 = _context2['catch'](6);
            _didIteratorError = true;
            _iteratorError = _context2.t0;

          case 25:
            _context2.prev = 25;
            _context2.prev = 26;

            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }

          case 28:
            _context2.prev = 28;

            if (!_didIteratorError) {
              _context2.next = 31;
              break;
            }

            throw _iteratorError;

          case 31:
            return _context2.finish(28);

          case 32:
            return _context2.finish(25);

          case 33:
            

            tasks = ramda.compose(ramda.map(reduxSaga_effects.join), ramda.map(ramda.prop('task')), ramda.reject(ramda.prop('terminated')))(children);


            supervisor.debug('init: loop', { tasks: tasks });

            _context2.next = 38;
            return reduxSaga_effects.race(ramda.merge(tasks, { waitedLongEnough: reduxSaga_effects.call(reduxSaga.delay, maxT / maxR, true) }));

          case 38:
            waitedLongEnough = _context2.sent;

            if (ramda.has('waitedLongEnough', waitedLongEnough)) {
              _context2.next = 42;
              break;
            }

            supervisor.debug('init: not waited long enough', { waitedLongEnough: waitedLongEnough });
            throw new Error('Restart frequency limit reached');

          case 42:
            _context2.t1 = ramda.head;
            _context2.t2 = ramda.values;
            _context2.next = 46;
            return reduxSaga_effects.race(tasks);

          case 46:
            _context2.t3 = _context2.sent;
            _context2.t4 = (0, _context2.t2)(_context2.t3);
            terminatedId = (0, _context2.t1)(_context2.t4);


            supervisor.debug('init: %s terminated', terminatedId);

            if (!(strategy === ONE_FOR_MANY)) {
              _context2.next = 88;
              break;
            }

            // restart all workers
            _iteratorNormalCompletion2 = true;
            _didIteratorError2 = false;
            _iteratorError2 = undefined;
            _context2.prev = 54;
            _iterator2 = ramda.values(children)[Symbol.iterator]();

          case 56:
            if (_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done) {
              _context2.next = 71;
              break;
            }

            child = _step2.value;

            if (!(child.id !== terminatedId)) {
              _context2.next = 61;
              break;
            }

            _context2.next = 61;
            return reduxSaga_effects.cancel(child.task);

          case 61:
            if (!(child.restart === PERMANENT)) {
              _context2.next = 67;
              break;
            }

            _context2.next = 64;
            return reduxSaga_effects.fork(child.start);

          case 64:
            children[child.id].task = _context2.sent;
            _context2.next = 68;
            break;

          case 67:
            children[child.id].terminated = true;

          case 68:
            _iteratorNormalCompletion2 = true;
            _context2.next = 56;
            break;

          case 71:
            _context2.next = 77;
            break;

          case 73:
            _context2.prev = 73;
            _context2.t5 = _context2['catch'](54);
            _didIteratorError2 = true;
            _iteratorError2 = _context2.t5;

          case 77:
            _context2.prev = 77;
            _context2.prev = 78;

            if (!_iteratorNormalCompletion2 && _iterator2.return) {
              _iterator2.return();
            }

          case 80:
            _context2.prev = 80;

            if (!_didIteratorError2) {
              _context2.next = 83;
              break;
            }

            throw _iteratorError2;

          case 83:
            return _context2.finish(80);

          case 84:
            return _context2.finish(77);

          case 85:
            return _context2.abrupt('continue', 33);

          case 88:
            if (!(children[terminatedId].restart === PERMANENT)) {
              _context2.next = 94;
              break;
            }

            _context2.next = 91;
            return reduxSaga_effects.fork(children[terminatedId].start);

          case 91:
            children[terminatedId].task = _context2.sent;
            _context2.next = 95;
            break;

          case 94:
            children[terminatedId].terminated = true;

          case 95:
            return _context2.abrupt('continue', 33);

          case 96:
            _context2.next = 33;
            break;

          case 98:
          case 'end':
            return _context2.stop();
        }
      }
    }, init, this, [[6, 21, 25, 33], [26,, 28, 32], [54, 73, 77, 85], [78,, 80, 84]]);
  })
};

/**
 * Testing utils
 *
 * @namespace TestUtils
 *
 */

var resumeIteratorAndCheckOutput = function resumeIteratorAndCheckOutput(t, iterator) {
  return function (into, outof) {
    var value = void 0;
    if (!iterator) {
      throw 'iterator is undefined';
    }

    try {
      value = iterator.next(into).value;
      t.deepEqual(value, outof);
    } catch (e) {
      t.deepEqual(e.message, outof.error.message);
    }
  };
};

/**
 * checks generator scenario
 *
 * this is equivalent to:
 * asserting that two iterators are equal, when given the same input
 *
 * @param {NodeTap} t node-tap instance
 * @param {Iterator} iter
 * @param {[]} scenario Scenario data
 * @returns {void}
 *
 * @memberof TestUtils
 */
var checkGeneratorScenario = function checkGeneratorScenario(t, iter, scenario) {
  var next = resumeIteratorAndCheckOutput(t, iter);

  for (var i = 0; i < scenario.length - 1; i += 2) {
    var input = scenario[i];
    var output = scenario[i + 1];
    next(input, output);
  }
};

exports.ONE_FOR_ONE = ONE_FOR_ONE;
exports.ONE_FOR_MANY = ONE_FOR_MANY;
exports.PERMANENT = PERMANENT;
exports.TEMPORARY = TEMPORARY;
exports.supervisor = supervisor;
exports.resumeIteratorAndCheckOutput = resumeIteratorAndCheckOutput;
exports.checkGeneratorScenario = checkGeneratorScenario;
//# sourceMappingURL=index.js.map
